%
%   LTSERVO.M   Simulation and sensitivity analysis of light tracking servo 
%               mechanism with proportional control
% 
%   Original code by Paul Batcheller (Aug 1992)
%   Minor modifications over the years by J. R. White (last update: Feb. 2020)
%

      clear all,   close all
%  
%   MOTOR CHARACTERISTICS (usually given with spec sheet or measured)
%   Ra - Resistance of the motor armature (ohms)
%   La - Inductance of the motor armature (H)
%   kv - velocity constant (Vsec/rad)
%   kt - torque constant (Nm/A)
      Ra = 1.75;      La = 2.83e-3;      kv = 0.093;      kt = 0.0924;
%
%   SYSTEM CHARACTERISTICS (study sensitivity of system dynamics to these parameters)
%   Jeq - Inertia seen by the motor.  Includes inertia of the load. (kg m^2)
%   Beq   - Mechanical damping coefficient associated with rotation. (Nms)
%   K - Gain of the photodiode circuit (volts/radian)
      Jeq = 30.0e-6;      Beq = 5.0e-3;
%
%   Create root locus plot to find default for reference gain K
%   (work with open loop transfer function)
      num = kt/(La*Jeq);
      d2 = Ra/La + Beq/Jeq;        d3 = (Ra*Beq + kt*kv)/(La*Jeq);    
      den = [1 d2 d3 0];
%
      figure(1)
      sys = tf(num,den);  rlocus(sys),sgrid
      axis([-500 50 -500 500])     
      hd = findobj(gcf,'LineStyle','-'); nhd = length(hd); 
      for i = 1:nhd; set(hd(i),'LineWidth',2); end
%
%----------------------------------------------------------------
%   MAIN SENSITIVITY ANALYSIS (repeat as many times as desired) 
%----------------------------------------------------------------
%
      repeat = 'y';         
      while repeat == 'y'                   
      Jeq = 30.0e-6;   Beq = 5.0e-3;    K = 25.0;  
      n = menu('Select variable to analyze','GAIN Parameter','DAMPING Term', ...
               'INERTIA of System');  
%
%   DEVELOP THREE TRANSFER FUNCTIONS FOR DIFFERENT GAINS  
        if n == 1  
          opt = 'Sensitivity to Gain'
          for i = 1:3   
            K = input('Input gain [typical = 25 V/radian]: ');  
            d2 = Ra/La + Beq/Jeq; 
            d3 = (Ra*Beq + kt*kv)/(La*Jeq);     
            d4 = (K*kt)/(La*Jeq);          
            NUM(i,:) = (K*kt)/(La*Jeq);   
            DEN(i,:) = [1 d2 d3 d4];    
          end    
%
%   DEVELOP THREE TRANSFER FUNCTIONS FOR DIFFERENT MECHANICAL DAMPING   
        elseif n == 2        
          opt = 'Sensitivity to Damping'
          for i = 1:3      
            Beq = input('Input damping factor [typical = 5.0e-3 Nms]: ');
            d2 = Ra/La + Beq/Jeq;       
            d3 = (Ra*Beq + kt*kv)/(La*Jeq);  
            d4 = (K*kt)/(La*Jeq);      
            NUM(i,:) = (K*kt)/(La*Jeq);    
            DEN(i,:) = [1 d2 d3 d4];    
          end           
%
%   DEVELOP THREE TRANSFER FUNCTIONS FOR DIFFERENT INERTIAS  
        elseif n == 3    
          opt = 'Sensitivity to Inertia'
          for i = 1:3    
            Jeq = input('Input inertia [typical = 30.0e-6 kg m^2]: '); 
            d2 = Ra/La + Beq/Jeq;              
            d3 = (Ra*Beq + kt*kv)/(La*Jeq);   
            d4 = (K*kt)/(La*Jeq);         
            NUM(i,:) = (K*kt)/(La*Jeq);        
            DEN(i,:) = [1 d2 d3 d4];    
          end    
        end  
        sys1 = tf(NUM(1,:),DEN(1,:));    
        sys2 = tf(NUM(2,:),DEN(2,:));   
        sys3 = tf(NUM(3,:),DEN(3,:));
%
%----------------------------------------------------------------  
%   GENERATE BODE PLOTS  and  STEP RESPONSE
%----------------------------------------------------------------  
%
%   Get range of frequency       
        w1 = input('Select beginning freq. exponent (rad/sec) [1]: ');
        if isempty(w1);   w1=1;   end
        w2 = input('Select end freq. exponent (rad/sec) [3]: ');
        if isempty(w2);   w2=3;   end
        w = logspace(w1,w2,100);
%
%   Bode plots of closed loop system
        figure(2)
        bode(sys1,'r',sys2,'g--',sys3,'b:',w),grid
        legend('low','mid','high','Location','SouthWest');
        hd = findobj(gcf,'Color','r'); nhd = length(hd); 
        for i = 1:nhd; set(hd(i),'LineWidth',2); end
        hd = findobj(gcf,'LineStyle','--'); nhd = length(hd); 
        for i = 1:nhd; set(hd(i),'LineWidth',2); end
        hd = findobj(gcf,'Color','b'); nhd = length(hd); 
        for i = 1:nhd; set(hd(i),'LineWidth',2); end
        title(['Frequency Response -- ', opt]);
%
%   Step response of closed loop system
        t2 = input('Specify end time (sec) [0.08]: ');
        if isempty(t2);    t2=0.08;     end
        t = linspace(0,t2,101);
        xs(:,1) = step(sys1,t);
        xs(:,2) = step(sys2,t);
        xs(:,3) = step(sys3,t);
%
%   plot step response
        figure(3)
        plot(t,xs(:,1),'r-',t,xs(:,2),'g--',t,xs(:,3),'b:','LineWidth',2),grid
        xlabel('time (sec)'),ylabel('position (radians)');
        title(['Step Response -- ', opt]);
        legend('low','mid','high');
%
        repeat = input('Do again? y/n [n]: ','s');
        if isempty(repeat);   repeat = 'n';    end
      end      % end of while repeat
%
%   normal termination
